var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { useCallback, useRef } from 'react';
import { ConcurrencyError } from './types';
import { useLatest } from '../generic/useLatest';
import { areArgsEqual } from '../utils';
/**
 * Joins multiple promise call into one, only one promise will be running
 * at-a-time, calls are not queued but rather joined. When promises are
 * called with arguments if arguments are the same as the original call
 * the promise will be joined and wait for the completion, if arguments
 * are different the promise will throw a concurrency exception.
 *
 * (?) Arguments are shallow compared with Object.is
 * (same strategy that is used by React on effect dependencies)
 *
 * (!) Task should be stable, changing task while a call is in progress
 * will not affect the outcome nor interrupt the execution.
 *
 * (?) Returned task is assured to be stable, an old result instance will
 * always call the most fresh task.
 *
 * todo : if fn includes states that change a sequence such is:
 *      call() -> setState() -> call() ...... return
 *      may return unexpected values to the the second call.
 *
 * /B 2022-10-19
 */
export function useSharedPromise(fn) {
    const lastFn = useLatest(fn);
    const stack = useRef();
    return useCallback((...args) => __awaiter(this, void 0, void 0, function* () {
        return new Promise((res, rej) => {
            var _a;
            if (!stack.current) {
                // First execution
                stack.current = { args, res: [res], rej: [rej] };
                lastFn.current.apply(undefined, args)
                    .then((r) => {
                    var _a;
                    (_a = stack.current) === null || _a === void 0 ? void 0 : _a.res.forEach(f => f(r));
                    stack.current = undefined;
                })
                    .catch((e) => {
                    var _a;
                    (_a = stack.current) === null || _a === void 0 ? void 0 : _a.rej.forEach(f => f(e));
                    stack.current = undefined;
                });
            }
            else {
                // Join or fail
                if (!areArgsEqual((_a = stack.current) === null || _a === void 0 ? void 0 : _a.args, args)) {
                    rej(new ConcurrencyError(`Promise already running with different arguments`));
                }
                else {
                    stack.current.res.push(res);
                    stack.current.rej.push(rej);
                }
            }
        });
    }), []);
}
