import { useEffect, useRef } from 'react';
/**
 * A hook that handles creation and destruction of global hidden tags. This hook
 * can be used to create dom elements that are outside the React scope. Used
 * by other hooks that provide macro-behaviours such are download and file select
 *
 * @param tagName tag type
 * @param init tag init function
 * @param options tag creation options
 * @param container tag container (defaults to body)
 */
export function useGlobalTag(tagName, init, options, container = document.body) {
    // Ref to the tag
    const ref = useRef();
    useEffect(() => {
        // Create and append to outer body
        ref.current = document.createElement(tagName, options);
        ref.current.style.display = 'none';
        container.appendChild(ref.current);
        init === null || init === void 0 ? void 0 : init(ref.current);
        // Remove the tag upon destruction
        return () => {
            if (!ref.current)
                return;
            container.removeChild(ref.current);
            ref.current = undefined;
        };
    }, []);
    return ref;
}
